package org.ajax4jsf.test.base;

import org.ajax4jsf.javascript.ScriptUtils;
import org.testng.Assert;

import com.thoughtworks.selenium.DefaultSelenium;

/**
 * The base java class for selenium tests implementation.
 * 
 * @author Andrey Markavtsov
 * 
 */
public abstract class SeleniumTestBase {

    /** Specifies the time to wait for page rendering */
    private static final Integer pageRenderTime = 5000;
    
    /** Specifies the time to wait for ajax processing */
    protected static final int ajaxCompletionTime = 3000;
    
    private static final String WINDOW_JS_RESOLVER = "selenium.browserbot.getCurrentWindow().";

    /** The default selenium instance */
    public DefaultSelenium selenium;

    /** Host */
    public String host;

    /** Port */
    public String port;

    /** Protocol */
    public String protocol;

    /** Defines the name of current j2ee application name */
    public static final String APPLICATION_NAME = "seleniumTest";

    public static final String COMPONENT_PREFIX_INSIDE_TABLE = "_Selenium_Test_Form1:_Selenium_Test_DataTable:0:";

    public static final String COMPONENT_PREFIX_INSIDE_PANEL = "_Selenium_Test_Form2:";

    public SeleniumTestBase(String protocol, String host, String port) {
        this.host = host;
        this.port = port;
        this.protocol = protocol;
    }

    /**
     * @param url
     * @param browser
     * @return
     */
    private DefaultSelenium createSeleniumClient(String url, String browser) {
        return new DefaultSelenium(host, 4444, browser, url);
    }

    /**
     * This method are invoking before selenium tests started
     */
    protected void startSelenium(String browser) {
    		selenium = createSeleniumClient(protocol + "://" + host + ":" + port + "/", browser);
    		selenium.start();
    }

    /**
     * Renders page
     */
    protected void renderPage(String homePage) {
        selenium.open(protocol + "://" + host + ":" + port + "/" + APPLICATION_NAME + homePage);
        selenium.waitForPageToLoad(String.valueOf(pageRenderTime));

        checkPageRendering(); // At the first we check if page has been
                                // rendered
        checkJSError(); // At the second we check if JS errors occurred

        reRenderForm(); // ReRender component

        checkPageRendering(); // Check all again
        checkJSError();

    }

    /**
     * Return component id inside Data table
     * 
     * @param id
     * @return
     */
    public String getComponentIdInsideTable(String id) {
        return COMPONENT_PREFIX_INSIDE_TABLE + id;
    }

    /**
     * Return component id inside Panel
     * 
     * @param id
     * @return
     */
    public String getComponentIdInsidePanel(String id) {
        return COMPONENT_PREFIX_INSIDE_PANEL + id;
    }

    /**
     * ReRenders the component
     */
    public void reRenderForm() {
        selenium.getEval("selenium.browserbot.getCurrentWindow().reRenderAll();");
        // clickById("_Selenium_Test_ReRender_Form:_reRender");
        waitForAjaxCompletion(3000);
    }
    
    
    /**
     * Performs script defined in parameter
     * @param script
     * @return
     */
    public String runScript(String script) {
    	String result = selenium.getEval(WINDOW_JS_RESOLVER + script);
    	checkJSError();
    	return result;
    }

    /**
     * This method are invoking after selenium tests completed
     */
     protected void stopSelenium() {
        selenium.stop();
        selenium = null;
    }

     /**
      * Checks if JS error occurred. Fails test if yes. This method should be
      * invoked after each event or any thing which can be a cause of JS error.
      */
     public void checkJSError() {
         String error = selenium.getEval(WINDOW_JS_RESOLVER + "checkError();");
         if (error != null && !("null".equals(error)) && !("".equals(error))) {
             Assert.fail("Failure by the following Javascript error: " + error);
         }
     }

     /**
      * Checks if page containing component test has been rendered completely
      */
     public void checkPageRendering() {
     	try {
     		String t1 = getTextById("_Selenium_Test_ControlPoint1");
     		String t2 = getTextById("_Selenium_Test_ControlPoint2");
     		if (t1 == null || t2 == null || !"Control1".equals(t1) || !"Control2".equals(t2)) {
     			Assert.fail("The page has been not rendered properlly");
     		}
     	}catch (Exception e) {
     		Assert.fail("The page has not been rendered properly due the following error: " + e);
 		}
     }
     /**
      * Waits while AJAX request will be completed
      * 
      * @param miliseconds -
      *                time to wait
      */
     public void waitForAjaxCompletion(int miliseconds) {
         selenium.waitForCondition(WINDOW_JS_RESOLVER + "done==true", String.valueOf(miliseconds));
     }
     
     /**
      * Waits while AJAX request will be completed
      * 
      * @param miliseconds -
      *                time to wait
      */
     public void waitForAjaxCompletion() {
     	waitForAjaxCompletion(ajaxCompletionTime);
     }

    
    /**
     * Waits while simple request will be completed
     * 
     * @param miliseconds -
     *                time to wait
     */
    public void waitForPageToLoad(int miliseconds) {
        selenium.waitForPageToLoad(String.valueOf(miliseconds));
    }
    
    /**
     * Waits while simple request will be completed
     * 
     */
    public void waitForPageToLoad() {
    	selenium.waitForPageToLoad(String.valueOf(pageRenderTime));
    }

    /**
     * Asserts DOM node value equals to value defined
     * 
     * @param id -
     *                DOM element id
     * @param value -
     *                value defined
     */
    public void AssertValueEquals(String id, String value) {
        String _v = getValueById(id);
        Assert.assertEquals(_v, value);
    }

    /**
     * Asserts DOM node value does not equal to value defined
     * 
     * @param id -
     *                DOM element id
     * @param value -
     *                value defined
     */
    public void AssertValueNotEquals(String id, String value) {
        String _v = getValueById(id);
        Assert.assertNotSame(_v, value);
    }

    /**
     * Asserts DOM node text equals to text defined
     * 
     * @param id -
     *                DOM element id
     * @param value -
     *                text defined
     */
    public void AssertTextEquals(String id, String value) {
        String _v = getTextById(id);
        Assert.assertEquals(_v, value);
    }

    /**
     * Asserts DOM node text does not equal to text defined
     * 
     * @param id -
     *                DOM element id
     * @param value -
     *                text defined
     */
    public void AssertTextNotEquals(String id, String value) {
        String _v = getTextById(id);
        Assert.assertNotSame(_v, value);
    }

    /**
     * Returns element's text
     * 
     * @param id -
     *                DOM element id
     * @return
     */
    public String getTextById(String id) {
        return selenium.getText("id=" + id);
    }

    /**
     * Returns element's value
     * 
     * @param id -
     *                DOM element id
     * @return
     */
    public String getValueById(String id) {
        return selenium.getValue("id=" + id);
    }
    
    
    /**
     * Returns element's width
     * @param id - DOM element id
     * @return
     */
    public Number getWidthById(String id) {
    	return selenium.getElementWidth("id=" + id);
    }

    /**
     * Clicks on element
     * 
     * @param id -
     *                DOM element id
     * @return
     */
    public void clickById(String id) {
        selenium.click("id=" + id);
        checkJSError();
    }

    /**
     * Return true if element is visible
     * @param id - DOM element id
     * @return
     */
    public boolean isVisibleById(String id) {
        return selenium.isVisible("id=" + id);
    }
    
    /**
     * Invokes JS method on client.
     * @param id         - DOM id of component 
     * @param method     - string method name
     * @param parameters - parameters
     * @return
     */
    public String invokeFromComponent(String id, String method, Object parameters) {
    	String _return = null;
    	StringBuffer buffer = new StringBuffer();
    	buffer.append("$('");
    	buffer.append(id);
    	buffer.append("').component.");
    	buffer.append(method);
    	buffer.append("(");
    	buffer.append(ScriptUtils.toScript(parameters));
    	buffer.append(");");
    	_return = runScript(buffer.toString());
    	return _return;
    }


    /**
     * Creates delay 
     * @param miliSeconds
     * @throws InterruptedException
     */
    public void delay(int miliSeconds) {
		try {
			Thread.sleep(miliSeconds);
		} catch (Exception e) {
			
		}
	}
      
    /**
     * Creates pause for defined time in miliSeconds 
     * @param miliSeconds
     */
    public void pause(int miliSeconds, String id) {
    	StringBuffer script = new StringBuffer("pause(");
    	script.append(miliSeconds);
    	script.append(",'");
    	script.append(id);
    	script.append("');");
    	runScript(script.toString());
    	selenium.waitForCondition(WINDOW_JS_RESOLVER + "pauseHolder['" + id + "'] == true;", String.valueOf(miliSeconds + 1000));
    }

}
